@@ -2,7 +2,6 @@
 //! Please refer to the other chapters of the help for documentation of the functions.
 use super::*;
 use basic_dsp_vector::*;
-use basic_dsp_vector::window_functions::*;
 use basic_dsp_vector::conv_types::*;
 use num_complex::*;
 use std::slice;
@@ -46,8 +45,7 @@ pub extern "C" fn new_with_performance_options32(is_complex: i32,
                                                  init_value: f32,
                                                  length: usize,
                                                  delta: f32,
-                                                 core_limit: usize,
-                                                 early_temp_allocation: bool)
+                                                 core_limit: usize)
                                                  -> VecBox {
     let domain = if domain == 0 {
         DataDomain::Time
@@ -60,7 +58,7 @@ pub extern "C" fn new_with_performance_options32(is_complex: i32,
         buffer: SingleBuffer::new(),
     });
     vector.vec.set_delta(delta);
-    vector.vec.set_multicore_settings(MultiCoreSettings::new(core_limit, early_temp_allocation));
+    vector.vec.set_multicore_settings(MultiCoreSettings::new(core_limit));
     vector
 
 }
@@ -149,7 +147,7 @@ pub extern "C" fn real_dot_product32(vector: &VecBuf,
                                      operand: &VecBuf)
                                      -> ScalarInteropResult<f32> {
     vector.convert_scalar(|v| {
-        DotProductOps::<f32, GenDspVec<Vec<f32>, f32>>::dot_product(v, &operand.vec)
+        DotProductOps::<GenDspVec<Vec<f32>, f32>, f32, f32, RealOrComplexData, TimeOrFrequencyData>::dot_product(v, &operand.vec)
     },
     0.0)
 }
@@ -159,19 +157,21 @@ pub extern "C" fn complex_dot_product32(vector: &VecBuf,
                                         operand: &VecBuf)
                                         -> ScalarInteropResult<Complex32> {
     vector.convert_scalar(|v| {
-        DotProductOps::<Complex32, GenDspVec<Vec<f32>, f32>>::dot_product(v, &operand.vec)
+        DotProductOps::<GenDspVec<Vec<f32>, f32>, Complex32, f32, RealOrComplexData, TimeOrFrequencyData>::dot_product(v, &operand.vec)
     },
     Complex32::new(0.0, 0.0))
 }
 
 #[no_mangle]
 pub extern "C" fn real_statistics32(vector: &VecBuf) -> Statistics<f32> {
-    vector.vec.statistics()
+    let vec = &vector.vec as &StatisticsOps<f32, Result = Statistics<f32>>;
+    vec.statistics()
 }
 
 #[no_mangle]
 pub extern "C" fn complex_statistics32(vector: &VecBuf) -> Statistics<Complex32> {
-    vector.vec.statistics()
+    let vec = &vector.vec as &StatisticsOps<Complex32, Result = Statistics<Complex32>>;
+    vec.statistics()
 }
 
 #[no_mangle]
@@ -195,32 +195,34 @@ pub extern "C" fn complex_sum_sq32(vector: &VecBuf) -> Complex32 {
 }
 #[no_mangle]
 pub extern "C" fn real_dot_product_prec32(vector: &VecBuf,
-                                     operand: &VecBuf)
-                                     -> ScalarInteropResult<f32> {
+                                          operand: &VecBuf)
+                                          -> ScalarInteropResult<f32> {
     vector.convert_scalar(|v| {
-        PreciseDotProductOps::<f32, GenDspVec<Vec<f32>, f32>>::dot_product_prec(v, &operand.vec)
+        PreciseDotProductOps::<GenDspVec<Vec<f32>, f32>, f32, f32, RealOrComplexData, TimeOrFrequencyData>::dot_product_prec(v, &operand.vec)
     },
     0.0)
 }
 
 #[no_mangle]
 pub extern "C" fn complex_dot_product_prec32(vector: &VecBuf,
-                                        operand: &VecBuf)
-                                        -> ScalarInteropResult<Complex32> {
+                                             operand: &VecBuf)
+                                             -> ScalarInteropResult<Complex32> {
     vector.convert_scalar(|v| {
-        PreciseDotProductOps::<Complex32, GenDspVec<Vec<f32>, f32>>::dot_product_prec(v, &operand.vec)
+        PreciseDotProductOps::<GenDspVec<Vec<f32>, f32>, Complex32, f32, RealOrComplexData, TimeOrFrequencyData>::dot_product_prec(v, &operand.vec)
     },
     Complex32::new(0.0, 0.0))
 }
 
 #[no_mangle]
 pub extern "C" fn real_statistics_prec32(vector: &VecBuf) -> Statistics<f64> {
-    vector.vec.statistics_prec()
+    let vec = &vector.vec as &PreciseStatisticsOps<f64, Result = Statistics<f64>>;
+    vec.statistics_prec()
 }
 
 #[no_mangle]
 pub extern "C" fn complex_statistics_prec32(vector: &VecBuf) -> Statistics<Complex64> {
-    vector.vec.statistics_prec()
+    let vec = &vector.vec as &PreciseStatisticsOps<Complex64, Result = Statistics<Complex64>>;
+    vec.statistics_prec()
 }
 
 #[no_mangle]
@@ -255,7 +257,7 @@ pub extern "C" fn zero_pad32(vector: Box<VecBuf>,
                              padding_option: i32)
                              -> VectorInteropResult<VecBuf> {
     let padding_option = translate_to_padding_option(padding_option);
-    vector.convert_vec(|v, b| Ok(v.zero_pad_b(b, points, padding_option)))
+    vector.convert_vec(|v, b| v.zero_pad_b(b, points, padding_option))
 }
 
 #[no_mangle]
@@ -512,14 +514,16 @@ pub extern "C" fn to_imag32(vector: Box<VecBuf>) -> VectorInteropResult<VecBuf>
 pub extern "C" fn map_inplace_real32(vector: Box<VecBuf>,
                                      map: extern "C" fn(f32, usize) -> f32)
                                      -> VectorInteropResult<VecBuf> {
-    vector.convert_vec(|v, _| Ok(v.map_inplace((), move |v, i, _| map(v, i))))
+    let map = move |v, i, _| map(v, i);
+    vector.convert_vec(|v, _| Ok(v.map_inplace((), &map)))
 }
 
 #[no_mangle]
 pub extern "C" fn map_inplace_complex32(vector: Box<VecBuf>,
                                         map: extern "C" fn(Complex32, usize) -> Complex32)
                                         -> VectorInteropResult<VecBuf> {
-    vector.convert_vec(|v, _| Ok(v.map_inplace((), move |v, i, _| map(v, i))))
+    let map = move |v, i, _| map(v, i);
+    vector.convert_vec(|v, _| Ok(v.map_inplace((), &map)))
 }
 
 /// Warning: This function interface heavily works around the Rust type system and the safety
@@ -531,14 +535,13 @@ pub extern "C" fn map_aggregate_real32(vector: &VecBuf,
                                                                 -> *const c_void)
                                        -> ScalarResult<*const c_void> {
     unsafe {
-        let result = vector.convert_scalar(|v| {
-            v.map_aggregate((),
-                            move |v, i, _| mem::transmute(map(v, i)),
-                            move |a: usize, b: usize| {
-                                mem::transmute(aggregate(mem::transmute(a), mem::transmute(b)))
-                            })
-        },
-                                           mem::transmute(0usize));
+        let map = move |v, i, _| mem::transmute(map(v, i));
+        let aggr = move |a: usize, b: usize| {
+            mem::transmute(aggregate(mem::transmute(a), mem::transmute(b)))
+        };
+
+        let result =
+            vector.convert_scalar(|v| v.map_aggregate((), &map, &aggr), mem::transmute(0usize));
         mem::transmute(result)
     }
 }
@@ -552,14 +555,13 @@ pub extern "C" fn map_aggregate_complex32(vector: &VecBuf,
                                                                    -> *const c_void)
                                           -> ScalarResult<*const c_void> {
     unsafe {
-        let result = vector.convert_scalar(|v| {
-            v.map_aggregate((),
-                            move |v, i, _| mem::transmute(map(v, i)),
-                            move |a: usize, b: usize| {
-                                mem::transmute(aggregate(mem::transmute(a), mem::transmute(b)))
-                            })
-        },
-                                           mem::transmute(0usize));
+        let map = move |v, i, _| mem::transmute(map(v, i));
+        let aggr = move |a: usize, b: usize| {
+            mem::transmute(aggregate(mem::transmute(a), mem::transmute(b)))
+        };
+
+        let result =
+            vector.convert_scalar(|v| v.map_aggregate((), &map, &aggr), mem::transmute(0usize));
         mem::transmute(result)
     }
 }
@@ -747,59 +749,84 @@ pub extern "C" fn overwrite_data32(mut vector: Box<VecBuf>,
 
 #[no_mangle]
 pub extern "C" fn real_statistics_split32(vector: &VecBuf,
-                                             data: *mut Statistics<f32>,
-                                             len: usize)
-                                             -> i32 {
+                                          data: *mut Statistics<f32>,
+                                          len: usize)
+                                          -> i32 {
     let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
-    let stats = vector.vec.statistics_split(data.len());
-    for i in 0..stats.len() {
-        data[i] = stats[i];
-    }
+    let vec = &vector.vec as &StatisticsSplitOps<f32, Result = StatsVec<Statistics<f32>>>;
+    let stats = vec.statistics_split(data.len());
+    match stats {
+        Ok(s) => {
+            for i in 0..s.len() {
+                data[i] = s[i];
+            }
 
-    0
+            0
+        }
+        Err(r) => translate_error(r),
+    }
 }
 
 #[no_mangle]
 pub extern "C" fn complex_statistics_split32(vector: &VecBuf,
-                                                data: *mut Statistics<Complex32>,
-                                                len: usize)
-                                                -> i32 {
-    let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
-    let stats = vector.vec.statistics_split(data.len());
-    for i in 0..stats.len() {
-        data[i] = stats[i];
-    }
-
-    0
-}
-
-
-#[no_mangle]
-pub extern "C" fn real_statistics_splitted32(vector: &VecBuf,
-                                             data: *mut Statistics<f32>,
+                                             data: *mut Statistics<Complex32>,
                                              len: usize)
                                              -> i32 {
     let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
-    let stats = vector.vec.statistics_split(data.len());
-    for i in 0..stats.len() {
-        data[i] = stats[i];
-    }
+    let vec = &vector.vec as &StatisticsSplitOps<Complex32,
+                                                 Result = StatsVec<Statistics<Complex32>>>;
+    let stats = vec.statistics_split(data.len());
+    match stats {
+        Ok(s) => {
+            for i in 0..s.len() {
+                data[i] = s[i];
+            }
 
-    0
+            0
+        }
+        Err(r) => translate_error(r),
+    }
 }
 
 #[no_mangle]
-pub extern "C" fn complex_statistics_splitted32(vector: &VecBuf,
-                                                data: *mut Statistics<Complex32>,
-                                                len: usize)
-                                                -> i32 {
+pub extern "C" fn real_statistics_split_prec32(vector: &VecBuf,
+                                               data: *mut Statistics<f64>,
+                                               len: usize)
+                                               -> i32 {
     let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
-    let stats = vector.vec.statistics_split(data.len());
-    for i in 0..stats.len() {
-        data[i] = stats[i];
-    }
+    let vec = &vector.vec as &PreciseStatisticsSplitOps<f64, Result = StatsVec<Statistics<f64>>>;
+    let stats = vec.statistics_split_prec(data.len());
+    match stats {
+        Ok(s) => {
+            for i in 0..s.len() {
+                data[i] = s[i];
+            }
 
-    0
+            0
+        }
+        Err(r) => translate_error(r),
+    }
+}
+
+#[no_mangle]
+pub extern "C" fn complex_statistics_split_prec32(vector: &VecBuf,
+                                                  data: *mut Statistics<Complex64>,
+                                                  len: usize)
+                                                  -> i32 {
+    let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
+    let vec = &vector.vec as &PreciseStatisticsSplitOps<Complex64,
+                                                        Result = StatsVec<Statistics<Complex64>>>;
+    let stats = vec.statistics_split_prec(data.len());
+    match stats {
+        Ok(s) => {
+            for i in 0..s.len() {
+                data[i] = s[i];
+            }
+
+            0
+        }
+        Err(r) => translate_error(r),
+    }
 }
 
 #[no_mangle]
@@ -833,11 +860,11 @@ pub extern "C" fn mirror32(vector: Box<VecBuf>) -> VectorInteropResult<VecBuf> {
 }
 
 pub extern "C" fn fft_shift32(vector: Box<VecBuf>) -> VectorInteropResult<VecBuf> {
-    vector.convert_vec(|v, b| Ok(v.fft_shift(b)))
+    vector.convert_vec(|v, _| Ok(v.fft_shift()))
 }
 
 pub extern "C" fn ifft_shift32(vector: Box<VecBuf>) -> VectorInteropResult<VecBuf> {
-    vector.convert_vec(|v, b| Ok(v.ifft_shift(b)))
+    vector.convert_vec(|v, _| Ok(v.ifft_shift()))
 }
 
 /// `window` argument is translated to:
@@ -1027,10 +1054,10 @@ pub extern "C" fn correlate32(vector: Box<VecBuf>, other: &VecBuf) -> VectorInte
 }
 
 #[no_mangle]
-pub extern "C" fn convolve_vector32(vector: Box<VecBuf>,
+pub extern "C" fn convolve_signal32(vector: Box<VecBuf>,
                                     impulse_response: &VecBuf)
                                     -> VectorInteropResult<VecBuf> {
-    vector.convert_vec(|v, b| v.convolve_vector(b, &impulse_response.vec))
+    vector.convert_vec(|v, b| v.convolve_signal(b, &impulse_response.vec))
 }
 
 /// Convolves the vector with an impulse response defined by `impulse_response` and
@@ -1200,12 +1227,12 @@ pub extern "C" fn interpolatef32(vector: Box<VecBuf>,
 /// function at every call and can be used to store parameters.
 #[no_mangle]
 pub extern "C" fn interpolate_custom32(vector: Box<VecBuf>,
-                                        frequency_response: extern "C" fn(*const c_void, f32) -> f32,
-                                        frequency_response_data: *const c_void,
-                                        is_symmetric: bool,
-                                        dest_points: usize,
-                                        delay: f32)
-                                        -> VectorInteropResult<VecBuf> {
+                                       frequency_response: extern "C" fn(*const c_void, f32) -> f32,
+                                       frequency_response_data: *const c_void,
+                                       is_symmetric: bool,
+                                       dest_points: usize,
+                                       delay: f32)
+                                       -> VectorInteropResult<VecBuf> {
     unsafe {
         let function: &RealFrequencyResponse<f32> = &ForeignRealConvolutionFunction {
             conv_function: frequency_response,
@@ -1224,24 +1251,20 @@ pub extern "C" fn interpolate_custom32(vector: Box<VecBuf>,
 /// `rolloff` is only used if this is a valid parameter for the selected `impulse_response`
 #[no_mangle]
 pub extern "C" fn interpolate32(vector: Box<VecBuf>,
-                                 frequency_response: i32,
-                                 rolloff: f32,
-                                 dest_points: usize,
-                                 delay: f32)
-                                 -> VectorInteropResult<VecBuf> {
+                                frequency_response: i32,
+                                rolloff: f32,
+                                dest_points: usize,
+                                delay: f32)
+                                -> VectorInteropResult<VecBuf> {
     let function = translate_to_real_frequency_response(frequency_response, rolloff);
-    vector.convert_vec(|v, b| {
-        v.interpolate(b, Some(function.as_ref()), dest_points, delay)
-    })
+    vector.convert_vec(|v, b| v.interpolate(b, Some(function.as_ref()), dest_points, delay))
 }
 
 #[no_mangle]
 pub extern "C" fn interpft32(vector: Box<VecBuf>,
-                                 dest_points: usize)
-                                 -> VectorInteropResult<VecBuf> {
-    vector.convert_vec(|v, b| {
-        Ok(v.interpft(b, dest_points))
-    })
+                             dest_points: usize)
+                             -> VectorInteropResult<VecBuf> {
+    vector.convert_vec(|v, b| Ok(v.interpft(b, dest_points)))
 }
 
 /// Convolves the vector with an impulse response defined by `frequency_response` and
