@@ -3,7 +3,6 @@
 //! Please refer to the other chapters of the help for documentation of the functions.
 use super::*;
 use basic_dsp_vector::*;
-use basic_dsp_vector::window_functions::*;
 use basic_dsp_vector::conv_types::*;
 use num_complex::*;
 use std::slice;
@@ -47,8 +46,7 @@ pub extern "C" fn new_with_performance_options64(is_complex: i32,
                                                  init_value: f64,
                                                  length: usize,
                                                  delta: f64,
-                                                 core_limit: usize,
-                                                 early_temp_allocation: bool)
+                                                 core_limit: usize)
                                                  -> VecBox {
     let domain = if domain == 0 {
         DataDomain::Time
@@ -61,7 +59,7 @@ pub extern "C" fn new_with_performance_options64(is_complex: i32,
         buffer: SingleBuffer::new(),
     });
     vector.vec.set_delta(delta);
-    vector.vec.set_multicore_settings(MultiCoreSettings::new(core_limit, early_temp_allocation));
+    vector.vec.set_multicore_settings(MultiCoreSettings::new(core_limit));
     vector
 
 }
@@ -150,7 +148,7 @@ pub extern "C" fn real_dot_product64(vector: &VecBuf,
                                      operand: &VecBuf)
                                      -> ScalarInteropResult<f64> {
     vector.convert_scalar(|v| {
-        DotProductOps::<f64, GenDspVec<Vec<f64>, f64>>::dot_product(v, &operand.vec)
+        DotProductOps::<GenDspVec<Vec<f64>, f64>, f64, f64, RealOrComplexData, TimeOrFrequencyData>::dot_product(v, &operand.vec)
     },
     0.0)
 }
@@ -160,19 +158,21 @@ pub extern "C" fn complex_dot_product64(vector: &VecBuf,
                                         operand: &VecBuf)
                                         -> ScalarInteropResult<Complex64> {
     vector.convert_scalar(|v| {
-        DotProductOps::<Complex64, GenDspVec<Vec<f64>, f64>>::dot_product(v, &operand.vec)
+        DotProductOps::<GenDspVec<Vec<f64>, f64>, Complex64, f64, RealOrComplexData, TimeOrFrequencyData>::dot_product(v, &operand.vec)
     },
     Complex64::new(0.0, 0.0))
 }
 
 #[no_mangle]
 pub extern "C" fn real_statistics64(vector: &VecBuf) -> Statistics<f64> {
-    vector.vec.statistics()
+    let vec = &vector.vec as &StatisticsOps<f64, Result = Statistics<f64>>;
+    vec.statistics()
 }
 
 #[no_mangle]
 pub extern "C" fn complex_statistics64(vector: &VecBuf) -> Statistics<Complex64> {
-    vector.vec.statistics()
+    let vec = &vector.vec as &StatisticsOps<Complex64, Result = Statistics<Complex64>>;
+    vec.statistics()
 }
 
 #[no_mangle]
@@ -196,32 +196,34 @@ pub extern "C" fn complex_sum_sq64(vector: &VecBuf) -> Complex64 {
 }
 #[no_mangle]
 pub extern "C" fn real_dot_product_prec64(vector: &VecBuf,
-                                     operand: &VecBuf)
-                                     -> ScalarInteropResult<f64> {
+                                          operand: &VecBuf)
+                                          -> ScalarInteropResult<f64> {
     vector.convert_scalar(|v| {
-        PreciseDotProductOps::<f64, GenDspVec<Vec<f64>, f64>>::dot_product_prec(v, &operand.vec)
+        PreciseDotProductOps::<GenDspVec<Vec<f64>, f64>, f64, f64, RealOrComplexData, TimeOrFrequencyData>::dot_product_prec(v, &operand.vec)
     },
     0.0)
 }
 
 #[no_mangle]
 pub extern "C" fn complex_dot_product_prec64(vector: &VecBuf,
-                                        operand: &VecBuf)
-                                        -> ScalarInteropResult<Complex64> {
+                                             operand: &VecBuf)
+                                             -> ScalarInteropResult<Complex64> {
     vector.convert_scalar(|v| {
-        PreciseDotProductOps::<Complex64, GenDspVec<Vec<f64>, f64>>::dot_product_prec(v, &operand.vec)
+        PreciseDotProductOps::<GenDspVec<Vec<f64>, f64>, Complex64, f64, RealOrComplexData, TimeOrFrequencyData>::dot_product_prec(v, &operand.vec)
     },
     Complex64::new(0.0, 0.0))
 }
 
 #[no_mangle]
 pub extern "C" fn real_statistics_prec64(vector: &VecBuf) -> Statistics<f64> {
-    vector.vec.statistics_prec()
+    let vec = &vector.vec as &PreciseStatisticsOps<f64, Result = Statistics<f64>>;
+    vec.statistics_prec()
 }
 
 #[no_mangle]
 pub extern "C" fn complex_statistics_prec64(vector: &VecBuf) -> Statistics<Complex64> {
-    vector.vec.statistics_prec()
+    let vec = &vector.vec as &PreciseStatisticsOps<Complex64, Result = Statistics<Complex64>>;
+    vec.statistics_prec()
 }
 
 #[no_mangle]
@@ -256,7 +258,7 @@ pub extern "C" fn zero_pad64(vector: Box<VecBuf>,
                              padding_option: i32)
                              -> VectorInteropResult<VecBuf> {
     let padding_option = translate_to_padding_option(padding_option);
-    vector.convert_vec(|v, b| Ok(v.zero_pad_b(b, points, padding_option)))
+    vector.convert_vec(|v, b| v.zero_pad_b(b, points, padding_option))
 }
 
 #[no_mangle]
@@ -513,14 +515,16 @@ pub extern "C" fn to_imag64(vector: Box<VecBuf>) -> VectorInteropResult<VecBuf>
 pub extern "C" fn map_inplace_real64(vector: Box<VecBuf>,
                                      map: extern "C" fn(f64, usize) -> f64)
                                      -> VectorInteropResult<VecBuf> {
-    vector.convert_vec(|v, _| Ok(v.map_inplace((), move |v, i, _| map(v, i))))
+    let map = move |v, i, _| map(v, i);
+    vector.convert_vec(|v, _| Ok(v.map_inplace((), &map)))
 }
 
 #[no_mangle]
 pub extern "C" fn map_inplace_complex64(vector: Box<VecBuf>,
                                         map: extern "C" fn(Complex64, usize) -> Complex64)
                                         -> VectorInteropResult<VecBuf> {
-    vector.convert_vec(|v, _| Ok(v.map_inplace((), move |v, i, _| map(v, i))))
+    let map = move |v, i, _| map(v, i);
+    vector.convert_vec(|v, _| Ok(v.map_inplace((), &map)))
 }
 
 /// Warning: This function interface heavily works around the Rust type system and the safety
@@ -532,14 +536,13 @@ pub extern "C" fn map_aggregate_real64(vector: &VecBuf,
                                                                 -> *const c_void)
                                        -> ScalarResult<*const c_void> {
     unsafe {
-        let result = vector.convert_scalar(|v| {
-            v.map_aggregate((),
-                            move |v, i, _| mem::transmute(map(v, i)),
-                            move |a: usize, b: usize| {
-                                mem::transmute(aggregate(mem::transmute(a), mem::transmute(b)))
-                            })
-        },
-                                           mem::transmute(0usize));
+        let map = move |v, i, _| mem::transmute(map(v, i));
+        let aggr = move |a: usize, b: usize| {
+            mem::transmute(aggregate(mem::transmute(a), mem::transmute(b)))
+        };
+
+        let result =
+            vector.convert_scalar(|v| v.map_aggregate((), &map, &aggr), mem::transmute(0usize));
         mem::transmute(result)
     }
 }
@@ -553,14 +556,13 @@ pub extern "C" fn map_aggregate_complex64(vector: &VecBuf,
                                                                    -> *const c_void)
                                           -> ScalarResult<*const c_void> {
     unsafe {
-        let result = vector.convert_scalar(|v| {
-            v.map_aggregate((),
-                            move |v, i, _| mem::transmute(map(v, i)),
-                            move |a: usize, b: usize| {
-                                mem::transmute(aggregate(mem::transmute(a), mem::transmute(b)))
-                            })
-        },
-                                           mem::transmute(0usize));
+        let map = move |v, i, _| mem::transmute(map(v, i));
+        let aggr = move |a: usize, b: usize| {
+            mem::transmute(aggregate(mem::transmute(a), mem::transmute(b)))
+        };
+
+        let result =
+            vector.convert_scalar(|v| v.map_aggregate((), &map, &aggr), mem::transmute(0usize));
         mem::transmute(result)
     }
 }
@@ -748,59 +750,84 @@ pub extern "C" fn overwrite_data64(mut vector: Box<VecBuf>,
 
 #[no_mangle]
 pub extern "C" fn real_statistics_split64(vector: &VecBuf,
-                                             data: *mut Statistics<f64>,
-                                             len: usize)
-                                             -> i32 {
+                                          data: *mut Statistics<f64>,
+                                          len: usize)
+                                          -> i32 {
     let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
-    let stats = vector.vec.statistics_split(data.len());
-    for i in 0..stats.len() {
-        data[i] = stats[i];
-    }
+    let vec = &vector.vec as &StatisticsSplitOps<f64, Result = StatsVec<Statistics<f64>>>;
+    let stats = vec.statistics_split(data.len());
+    match stats {
+        Ok(s) => {
+            for i in 0..s.len() {
+                data[i] = s[i];
+            }
 
-    0
+            0
+        }
+        Err(r) => translate_error(r),
+    }
 }
 
 #[no_mangle]
 pub extern "C" fn complex_statistics_split64(vector: &VecBuf,
-                                                data: *mut Statistics<Complex64>,
-                                                len: usize)
-                                                -> i32 {
-    let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
-    let stats = vector.vec.statistics_split(data.len());
-    for i in 0..stats.len() {
-        data[i] = stats[i];
-    }
-
-    0
-}
-
-
-#[no_mangle]
-pub extern "C" fn real_statistics_splitted64(vector: &VecBuf,
-                                             data: *mut Statistics<f64>,
+                                             data: *mut Statistics<Complex64>,
                                              len: usize)
                                              -> i32 {
     let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
-    let stats = vector.vec.statistics_split(data.len());
-    for i in 0..stats.len() {
-        data[i] = stats[i];
-    }
+    let vec = &vector.vec as &StatisticsSplitOps<Complex64,
+                                                 Result = StatsVec<Statistics<Complex64>>>;
+    let stats = vec.statistics_split(data.len());
+    match stats {
+        Ok(s) => {
+            for i in 0..s.len() {
+                data[i] = s[i];
+            }
 
-    0
+            0
+        }
+        Err(r) => translate_error(r),
+    }
 }
 
 #[no_mangle]
-pub extern "C" fn complex_statistics_splitted64(vector: &VecBuf,
-                                                data: *mut Statistics<Complex64>,
-                                                len: usize)
-                                                -> i32 {
+pub extern "C" fn real_statistics_split_prec64(vector: &VecBuf,
+                                               data: *mut Statistics<f64>,
+                                               len: usize)
+                                               -> i32 {
     let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
-    let stats = vector.vec.statistics_split(data.len());
-    for i in 0..stats.len() {
-        data[i] = stats[i];
-    }
+    let vec = &vector.vec as &PreciseStatisticsSplitOps<f64, Result = StatsVec<Statistics<f64>>>;
+    let stats = vec.statistics_split_prec(data.len());
+    match stats {
+        Ok(s) => {
+            for i in 0..s.len() {
+                data[i] = s[i];
+            }
 
-    0
+            0
+        }
+        Err(r) => translate_error(r),
+    }
+}
+
+#[no_mangle]
+pub extern "C" fn complex_statistics_split_prec64(vector: &VecBuf,
+                                                  data: *mut Statistics<Complex64>,
+                                                  len: usize)
+                                                  -> i32 {
+    let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
+    let vec = &vector.vec as &PreciseStatisticsSplitOps<Complex64,
+                                                        Result = StatsVec<Statistics<Complex64>>>;
+    let stats = vec.statistics_split_prec(data.len());
+    match stats {
+        Ok(s) => {
+            for i in 0..s.len() {
+                data[i] = s[i];
+            }
+
+            0
+        }
+        Err(r) => translate_error(r),
+    }
 }
 
 #[no_mangle]
@@ -834,11 +861,11 @@ pub extern "C" fn mirror64(vector: Box<VecBuf>) -> VectorInteropResult<VecBuf> {
 }
 
 pub extern "C" fn fft_shift64(vector: Box<VecBuf>) -> VectorInteropResult<VecBuf> {
-    vector.convert_vec(|v, b| Ok(v.fft_shift(b)))
+    vector.convert_vec(|v, _| Ok(v.fft_shift()))
 }
 
 pub extern "C" fn ifft_shift64(vector: Box<VecBuf>) -> VectorInteropResult<VecBuf> {
-    vector.convert_vec(|v, b| Ok(v.ifft_shift(b)))
+    vector.convert_vec(|v, _| Ok(v.ifft_shift()))
 }
 
 /// `window` argument is translated to:
@@ -1028,10 +1055,10 @@ pub extern "C" fn correlate64(vector: Box<VecBuf>, other: &VecBuf) -> VectorInte
 }
 
 #[no_mangle]
-pub extern "C" fn convolve_vector64(vector: Box<VecBuf>,
+pub extern "C" fn convolve_signal64(vector: Box<VecBuf>,
                                     impulse_response: &VecBuf)
                                     -> VectorInteropResult<VecBuf> {
-    vector.convert_vec(|v, b| v.convolve_vector(b, &impulse_response.vec))
+    vector.convert_vec(|v, b| v.convolve_signal(b, &impulse_response.vec))
 }
 
 /// Convolves the vector with an impulse response defined by `impulse_response` and
@@ -1201,12 +1228,12 @@ pub extern "C" fn interpolatef64(vector: Box<VecBuf>,
 /// function at every call and can be used to store parameters.
 #[no_mangle]
 pub extern "C" fn interpolate_custom64(vector: Box<VecBuf>,
-                                        frequency_response: extern "C" fn(*const c_void, f64) -> f64,
-                                        frequency_response_data: *const c_void,
-                                        is_symmetric: bool,
-                                        dest_points: usize,
-                                        delay: f64)
-                                        -> VectorInteropResult<VecBuf> {
+                                       frequency_response: extern "C" fn(*const c_void, f64) -> f64,
+                                       frequency_response_data: *const c_void,
+                                       is_symmetric: bool,
+                                       dest_points: usize,
+                                       delay: f64)
+                                       -> VectorInteropResult<VecBuf> {
     unsafe {
         let function: &RealFrequencyResponse<f64> = &ForeignRealConvolutionFunction {
             conv_function: frequency_response,
@@ -1225,24 +1252,20 @@ pub extern "C" fn interpolate_custom64(vector: Box<VecBuf>,
 /// `rolloff` is only used if this is a valid parameter for the selected `impulse_response`
 #[no_mangle]
 pub extern "C" fn interpolate64(vector: Box<VecBuf>,
-                                 frequency_response: i32,
-                                 rolloff: f64,
-                                 dest_points: usize,
-                                 delay: f64)
-                                 -> VectorInteropResult<VecBuf> {
+                                frequency_response: i32,
+                                rolloff: f64,
+                                dest_points: usize,
+                                delay: f64)
+                                -> VectorInteropResult<VecBuf> {
     let function = translate_to_real_frequency_response(frequency_response, rolloff);
-    vector.convert_vec(|v, b| {
-        v.interpolate(b, Some(function.as_ref()), dest_points, delay)
-    })
+    vector.convert_vec(|v, b| v.interpolate(b, Some(function.as_ref()), dest_points, delay))
 }
 
 #[no_mangle]
 pub extern "C" fn interpft64(vector: Box<VecBuf>,
-                                 dest_points: usize)
-                                 -> VectorInteropResult<VecBuf> {
-    vector.convert_vec(|v, b| {
-        Ok(v.interpft(b, dest_points))
-    })
+                             dest_points: usize)
+                             -> VectorInteropResult<VecBuf> {
+    vector.convert_vec(|v, b| Ok(v.interpft(b, dest_points)))
 }
 
 /// Convolves the vector with an impulse response defined by `frequency_response` and
